using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace AAL77CheckHammingCodesAndPageParities
{
    //Version 1.0
    class Program
    {
        //Information about the format in which the FDR records the data can be found in the post 
        //at http://forums.randi.org/showpost.php?p=2022420&postcount=85 and in figure 4.2-2 of 
        //the document http://www.asc.gov.tw/author_files/ASC-TRT-99-007.pdf

        //The words in the figure are 16 bit words stored Least Significant Byte first and the 
        //Hamming Code & Page Parity occupies the last 11 bits of each 128 byte page. Within each 
        //16 bit word, the most significant bit is the first bit in the bit stream. 
        static void Main(string[] args)
        {
            FileStream inputFile = new FileStream(@"C:\Documents and Settings\Admin\My Documents\Pilots For 911 Truth\090324_0813\AA77 FDR\American 77.fdr", FileMode.Open, FileAccess.Read);
            long fileLength = inputFile.Length;
            long fileTextHeaderLength = 0;
            //Search through the text header at the beginning of the file until the first header 
            //or erased page is found.
            for (; fileTextHeaderLength < fileLength; fileTextHeaderLength += 2)
            {
                byte[] b = new byte[2];
                inputFile.Read(b, 0, 2);
                int w = (b[1] << 8) | b[0];
                if ((w == 0xFE6B) || (w == 0xFFFF))
                    break;
            }
            inputFile.Seek(fileTextHeaderLength, SeekOrigin.Begin);
            StreamWriter outputFile = new StreamWriter(@"C:\Documents and Settings\Admin\My Documents\Pilots For 911 Truth\AAL77 Hamming Codes and Page Parities.csv");
            outputFile.WriteLine("File Offset,Page Type,Erased Page,Hamming Code and Page Parity Correct,Calculated Hamming Code and Page Parity,File Hamming Code and Page Parity");
            long pageNum = 0;
            for (long filePos = fileTextHeaderLength; filePos < fileLength; filePos += 128)
            {
                byte[] page = new byte[128];
                inputFile.Read(page, 0, 128);
                string pageType = "";
                string hammingCodeAndPageParityCorrect = "";
                string calculatedHammingCodeAndPageParity = "";
                string fileHammingCodeAndPageParity = "";
                if ((pageNum++ & 0x1FF) < 2)
                    pageType = "Header";
                else
                {
                    pageType = "Flight Data";
                    int pageParity = 0;
                    int hammingCode = 0;
                    int shiftNum = 0;
                    //Look through each bit of the page except the page parity, one at a time.
                    for (int bitNum0 = 0; bitNum0 < 1023; bitNum0++)
                    {
                        int bitNum1 = bitNum0 + 1;
                        int bitHammingCode = (bitNum1 == (1 << shiftNum)) ? 1023 - shiftNum++ : bitNum1;
                        if (((page[(bitNum0 >> 3) ^ 1] << (bitNum0 & 7)) & 0x80) != 0)
                        {
                            //The bit numbered BitNum0 of the page is 1, so update the page Hamming code and page parity.
                            if (bitNum0 < 1013)
                                hammingCode ^= bitHammingCode;
                            pageParity ^= 1;
                        }
                    }
                    calculatedHammingCodeAndPageParity = ((hammingCode << 1) | pageParity).ToString("X");
                    fileHammingCodeAndPageParity = (((page[127] << 8) | page[126]) & 0x7FF).ToString("X");
                    hammingCodeAndPageParityCorrect = (calculatedHammingCodeAndPageParity == fileHammingCodeAndPageParity) ? "Correct" : "Incorrect";
                }
                //Check to see whether this page has been altered since it was erased (all bits set to 1)
                string erasedPage = "Erased";
                for (int i = 0; i < 128; i++)
                    if (page[i] != 0xFF)
                    {
                        erasedPage = "Filled";
                        break;
                    }
                outputFile.WriteLine("{0:X},{1},{2},{3},{4},{5}", filePos, pageType, erasedPage, hammingCodeAndPageParityCorrect, calculatedHammingCodeAndPageParity, fileHammingCodeAndPageParity);
            }
            outputFile.Close();
            inputFile.Close();
        }
    }
}
